#!/usr/bin/perl
# 
#  RC5LCD -- An LCDproc client to display the block-completion
#  	     status of RC5/DES.
#
#  LCDinfo -- Modified to work with the new RC5-72 client 
#             from Distributed.net
#
#  First Release: Version 0.01
#  Modified Release: Version 0.01.1
#
#  AUTHOR: Simmon Keith Barney <traevoli@usa.net>
#  MODIFIED: Martin Andrew <mayhem@linuxathome.net>
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

use POSIX;
use IO::Socket;
use Fcntl;

#setup variables
$display_info = 0;	# Set to 0 (zero) if you do not wish to see
			# program information screen (screen 4)

#sub routine for rounding to two decinmal places
sub round {
 my ($input) = @_;
 my $output = POSIX::floor ($input * 100.00);
 return ($output / 100);
}

if ($ARGV[0]){
  $logfile = $ARGV[0];
  print "Reading from $arg as logfile.\n";
} else {
  $logfile = "rc5.log";
  print "No logfile specified.  Using ./rc5.log\n";
}

# Connect to the server...
$remote = IO::Socket::INET->new(
		Proto     => "tcp",
		PeerAddr  => "localhost",
		PeerPort  => "13666",
	)
	|| die "Cannot connect to LCDproc port\n";

# Make sure our messages get there right away
$remote->autoflush(1);

`sleep 1`;	# Give server plenty of time to notice us...

print $remote "hello\n";
my $lcdconnect = <$remote>;
print $lcdconnect;

# Turn off blocking mode...
fcntl($remote, F_SETFL, O_NONBLOCK);

# Set up some screen widgets...
print $remote "client_set name {RC5-72 Status}\n";
print $remote "screen_add rc5buff\n";
print $remote "screen_set rc5buff name {RC5-72 Stats}\n";
print $remote "widget_add rc5buff title title\n";
print $remote "widget_set rc5buff title {RC5-72 Stats}\n";
print $remote "widget_add rc5buff one string\n";
print $remote "widget_add rc5buff two string\n";
print $remote "widget_add rc5buff three string\n";

print $remote "screen_add rc5info\n";
print $remote "screen_set rc5info name {RC5-72 Stats}\n";
print $remote "widget_add rc5info title title\n";
print $remote "widget_set rc5info title {RC5-72 Stats}\n";
print $remote "widget_add rc5info one string\n";
print $remote "widget_add rc5info two string\n";
print $remote "widget_add rc5info three string\n";

print $remote "screen_add rc5info2\n";
print $remote "screen_set rc5info2 name {RC5-72 Stats}\n";
print $remote "widget_add rc5info2 title title\n";
print $remote "widget_set rc5info2 title {RC5-72 Stats}\n";
print $remote "widget_add rc5info2 one string\n";
print $remote "widget_add rc5info2 two string\n";
print $remote "widget_add rc5info2 three string\n";

if ($display_info >= 1){
	print $remote "screen_add rc5lcd\n";
	print $remote "screen_set rc5lcd name {RC5-72 LCDinfo}\n";
	print $remote "widget_add rc5lcd title title\n";
	print $remote "widget_set rc5lcd title {RC5-72 LCDinfo}\n";
	print $remote "widget_add rc5lcd one string\n";
	print $remote "widget_add rc5lcd two string\n";
	print $remote "widget_add rc5lcd three string\n";
}

my ($machine, %down, %up, $i, $list);

#find how many crunchers have been started (only need to do this once)
$log = `cat $logfile`;
if ($log =~ /.*\s(\d*)\scruncher.*\sbeen started.*/s){
	$crunchers = $1;
}

while(1)
{
	while(defined($line = <$remote>)) {
	    print $line;
	}

	undef %down;
	undef %up;

### START PROCESSING CODE ###

	#get the last 10 lines from the log file
	$log = `tail --lines 10 $logfile`;

	#get input buffer contents (number of units)
	if ($log =~ /.*\sRC5-72:\s(\d+)\spacket.*\s.*.*buff-in.r72.*/s){
	  	$in = $1;
	}

	#get output buffer contents (number of units)
	if ($log =~ /.*\sRC5-72:\s(\d+)\spacket.*\s.*.*buff-out.r72.*/s){
	  	$out = $1;
	}

	#work out current buffer completion percentage
	if(($out == 0)) {
		$complete = "0.00";
	} else {
		$complete = (($out / ($in + $out)) * 100);
		$complete = round($complete);	#round to two decimal places
	}
        if(($in == 0) && ($out > 0)) {
        	$complete = "100.00";
	}

	#get the number of keys per second for last unit(s) processed
	if ($log =~ /.*\s-\s\[(.*)\skeys\/s\].*/s){
		$keys = $1;
	}
    
	#get the projected ideal time to completion
	if ($log =~ /.*Projected ideal time to completion:\s([0-9.:]*).*\n/s){
		$time = $1;
	}
	
	#get the cpu process time used for the dnetc client program
	$tmp = `ps ax | grep "dnetc"`;
	@tmp2 = split("\n", $tmp);
	if ($tmp2[0] =~ /.*\s([0-9:]*)\s.*dnetc.*/s) {
		$cpu_time = $1;
	}

	#print the results (screen 1)
	print $remote "widget_set rc5buff one 1 2 {Completed:  $out units}\n";
	print $remote "widget_set rc5buff two 1 3 {Remaining:  $in units}\n";
	print $remote "widget_set rc5buff three 1 4 {Percentage: $complete%}\n";
	  
	#print more stats (screen 2)
	print $remote "widget_set rc5info one 1 2 {No. Crunchers: $crunchers}\n";
	print $remote "widget_set rc5info two 1 3 {Keys/sec: $keys}\n";
	print $remote "widget_set rc5info three 1 4 {CPU Time: $cpu_time}\n";
	
	#print even more stats (screen 3)
	print $remote "widget_set rc5info2 one 1 2 {  Projected ideal}\n";
	print $remote "widget_set rc5info2 two 1 3 { time to completion}\n";
	print $remote "widget_set rc5info2 three 1 4 {   $time}\n";

	#print the author/coder information (screen 4) if configuration says "yes"
	if ($display_info >= 1){
		print $remote "widget_set rc5lcd one 1 2 {  Coded by Mayhem}\n";
		print $remote "widget_set rc5lcd two 1 3 {  Linuxathome.net}\n";
		print $remote "widget_set rc5lcd three 1 4 {lcd\@linuxathome.net}\n";
	}
    
	sleep 3;

### END PROCESSING CODE ###

}

close ($remote) || die "close: $!";
exit;
